package com.example.zxweb.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.example.zxweb.dto.ZxCelueDTO;
import com.example.zxweb.entity.ZxCelue;
import com.example.zxweb.exception.BusinessException;
import com.example.zxweb.mapper.ZxCelueMapper;
import com.example.zxweb.service.IZxCelueService;
import com.example.zxweb.utils.IotUtils;
import com.example.zxweb.vo.AddZxCeluePVO;
import com.example.zxweb.vo.QueryCelueVO;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @Description 策略信息
 * @Author ZhouWenTao
 * @Date 2023/9/13 14:09
 */
@Service
@Slf4j
public class ZxCelueServiceImpl extends ServiceImpl<ZxCelueMapper, ZxCelue> implements IZxCelueService {
    @Value("${iot.active}")
    private boolean active;
    @Override
    public boolean saveZxCelue(AddZxCeluePVO addZxCeluePVO) {
        return false;
    }

    @Override
    public boolean editStatus(ZxCelue zxCelue) {
        String status="0";
        try {
            ZxCelue celue = getById(zxCelue.getCelueid());
            LambdaUpdateWrapper<ZxCelue> updateWrapper=new LambdaUpdateWrapper<>();
            updateWrapper.eq(ZxCelue::getCelueid,celue.getCelueid());
            if ("0".equals(celue.getStatus())) {
                status="1";
                updateWrapper.set(ZxCelue::getStatus,status);
            }else{
                updateWrapper.set(ZxCelue::getStatus,status);
            }
            update(updateWrapper);
            //请求更改策略方法
            if (active) {
                //向网关发生更改状态指令
                IotUtils.editCelueStatus(celue.getCelueid(),status);
            }
        }catch (Exception e){
            throw new BusinessException(e);
        }
        return true;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean deleteZxCelue(String id) {
        //删除数据库中策略
        removeById(id);
        if (active) {
            //向网关发生删除指令
            IotUtils.deleteCelue(id);
        }

        return true;
    }

    @Override
    public IPage<ZxCelueDTO> pageList(QueryCelueVO queryCelueVO) {
        Integer page = queryCelueVO.getPage();
        Integer limit = queryCelueVO.getLimit();
        String name = queryCelueVO.getName();
        String status = queryCelueVO.getStatus();
        IPage<ZxCelueDTO> pageList = baseMapper.pageList(new Page<>(page, limit),queryCelueVO);
        for (ZxCelueDTO record : pageList.getRecords()) {
            if (StringUtils.isNotBlank(record.getDevid())) {
                List<String> collect = Arrays.stream(record.getDevid().split(",")).collect(Collectors.toList());
                record.setDevidList(collect);
            }
        }
        return pageList;
    }

    @Override
    public ZxCelueDTO getDetail(String id) {
        ZxCelue zxCelue = getById(id);

        ZxCelueDTO zxCelueDTO = new ZxCelueDTO();
        BeanUtils.copyProperties(zxCelue,zxCelueDTO);
        //拆分 所选设备id
        String devid = zxCelue.getDevid();
        if (StringUtils.isNotBlank(devid)) {
            List<String> collect = Arrays.stream(devid.split(",")).collect(Collectors.toList());
            zxCelueDTO.setDevidList(collect);
        }
        return zxCelueDTO;
    }

    public static void main(String[] args) {
        String timeType="day";
        String exeTime="08:30";
        String cron=null;
        String weeklyDay="1,2,3";//周一，周二，周三
        String day="1,31";
        String[] time = exeTime.split(":");
        Integer hour=Integer.parseInt(time[0]);//小时
        Integer minute=Integer.parseInt(time[1]);//分钟
        String startDate="2023-09-10";//开始时间
        String endDate="2023-09-30";//结束时间
        //每天
        if ("everyDay".equals(timeType)) {
            cron = String.format("0 %s %s * * ? *",minute,hour);
        } else if ("weekly".equals(timeType)) {
            //每周
            cron = String.format("0 %s %s ? * %s",minute,hour,weeklyDay);
        } else if("day".equals(timeType)){
            //单日
            cron = String.format("0 %s %s %s * ? ",minute,hour,day);
        } else if ("date".equals(timeType)){
            //日期段
            System.out.println("在"+startDate+"到"+endDate);
            cron = String.format("0 %s %s * * ? *",minute,hour);
        }
        System.out.println(cron);
    }
}
